var UserPrefsService = UserPrefsService || {};

//Creates a new cookie
UserPrefsService.createCookie = function createCookie(name, value, days) {
    if (days) {
        var date = new Date();
        date.setTime(date.getTime() + (days * 24 * 60 * 60 * 1000));
        var expires = "; expires=" + date.toGMTString();
    } else
        var expires = "";
    document.cookie = name + "=" + value + expires + "; path=/";
}

//Reads the cookie
UserPrefsService.readCookie = function readCookie(name) {
    var nameEQ = name + "=";
    var ca = document.cookie.split(';');
    for (var i = 0; i < ca.length; i++) {
        var c = ca[i];
        while (c.charAt(0) == ' ')
            c = c.substring(1, c.length);
        if (c.indexOf(nameEQ) == 0)
            return c.substring(nameEQ.length, c.length);
    }
    return "NA";
}

//erase the cookie
UserPrefsService.eraseCookie = function eraseCookie(name) {
    UserPrefsService.createCookie(name, "", -1);
}

//The gadget counter
UserPrefsService.incrementGadgetCount = function incrementGadgetCount() {
    var gadgetId = UserPrefsService.readCookie("nextGadgetId");
    var nextGadgetId = Number(gadgetId) + 1;
    UserPrefsService.createCookie("nextGadgetId", nextGadgetId, 0);
}

//The tab counter
UserPrefsService.incrementTabCount = function incrementGadgetCount() {
    var gadgetId = UserPrefsService.readCookie("nextTabId");
    var nextGadgetId = Number(gadgetId) + 1;
    UserPrefsService.createCookie("nextTabId", nextGadgetId, 0);
}

/**
 * Adds new gadgets. the new gadgets are stored in cookies
 * every gadget has a unique id a combination of "gadget" + next_available_gadgetId
 * Gadget layout is saved containing the tab id in it's name
 * the values in the layout are the variable gadget ID
 */
UserPrefsService.addGadget = function addGadget(userId, tabId, url,
                                                dashboardName, group) {
    var curGadgetId = UserPrefsService.storeGadget(url);

    var gadgetLayout = UserPrefsService.getGadgetLayout(userId, tabId, dashboardName);

    // Insert new gadget and save
    UserPrefsService.setGadgetLayout(userId, tabId, UserPrefsService.insertGadgetToLayout(gadgetLayout, curGadgetId), dashboardName);

    return true;
}

/**
 * Stores a given gadget URL and returns its unique ID for reference.
 *
 * @param url
 */
UserPrefsService.storeGadget = function storeGadget(url) {
    var curGadgetId = UserPrefsService.readCookie("nextGadgetId");
    if (curGadgetId == "NA") {
        curGadgetId = 0;
        var nextGadgetId = curGadgetId + 1;
        UserPrefsService.createCookie("nextGadgetId", nextGadgetId, 0);
    } else {
        UserPrefsService.incrementGadgetCount();
    }

    UserPrefsService.jsonBuilder("gadget-" + curGadgetId, url);

    return curGadgetId;
}

/**
 * Inserts a given gadget ID to an existing layout. Creates a layout using template, if one is not present.
 *
 * @param existingLayout
 * @param gadgetId
 */
UserPrefsService.insertGadgetToLayout = function (existingLayout, gadgetId) {
    if (existingLayout == "NA") {
        // Create a 3-column JSON layout and make this the first gadget
        existingLayout = layout3ColumnJSON;
    } else {
        existingLayout = JSON.parse(existingLayout);
    }

    var newGadgetElement = new Layout();
    newGadgetElement.type = "gadget";
    newGadgetElement.id = gadgetId;

    // Add this new gadget to column 1
    var column1LayoutELementsArray = existingLayout.layout[0].layout;
    column1LayoutELementsArray[column1LayoutELementsArray.length] = newGadgetElement;
    existingLayout.layout[0].layout = column1LayoutELementsArray;

    return JSON.stringify(existingLayout);
}

/**
 * Adds new tab to the layout
 * @param userId
 * @param tabTitle
 * @param dashboardName
 * @return current tab id
 */
UserPrefsService.addNewTab = function addNewTab(userId, tabTitle, dashboardName) {
    if (UserPrefsService.hasTabName(userId, tabTitle, dashboardName)) {
        return 0;
    }
    var curTabId = UserPrefsService.readCookie("nextTabId");
    if (curTabId == "NA") {
        curTabId = 1;
        var nextTabId = curTabId + 1;
        UserPrefsService.createCookie("nextTabId", nextTabId, 0);
    } else {
        UserPrefsService.incrementTabCount();
    }

    if (tabTitle == "") {
        tabTitle = "tab " + curTabId;
    }

    UserPrefsService.createCookie("tab-" + curTabId, tabTitle, 0);

    var tabLayout = UserPrefsService.readCookie("tabLayout");
    if (tabLayout == "NA") {
        tabLayout = curTabId;
        UserPrefsService.createCookie("tabLayout", tabLayout, 0);
    } else {
        tabLayout = tabLayout + "," + curTabId;
        UserPrefsService.createCookie("tabLayout", tabLayout, 0);
    }

    return curTabId;
}

UserPrefsService.getGadgetLayout = function getGadgetLayout(userId, tabId,
                                                            dashboardName) {
    var layoutArray = UserPrefsService.readCookie("gadgetLayout-" + tabId)
    if (layoutArray == '') {
        layoutArray = 'NA';
    }
    return layoutArray;
}

UserPrefsService.getGadgetPrefs = function getGadgetPrefs(userId, gadgetId,
                                                          prefId, dashboardName) {

    return UserPrefsService.readCookie("gadgetPref-" + gadgetId + "-" + prefId);

}

UserPrefsService.getGadgetUrlsToLayout = function getGadgetUrlsToLayout(userId,
                                                                        tabId, dashboardName) {

    try {
        var storedLayout = JSON.parse(UserPrefsService.getGadgetLayout("", tabId, ""));
    } catch(e) {
        // No point going forward.
        return "NA";
    }

    var gadgetIds = new Array();

    // Retrieve URLs corresponding to IDs in the JSON layout
    UserPrefsService.getUrlArrayFromLayout(storedLayout, gadgetIds);

    if (gadgetIds.length > 0) {
        var urlArray = new Array();

        for (var x = 0; x < gadgetIds.length; x++) {
            urlArray[urlArray.length] = UserPrefsService.getGadgetUrl("gadget-" + gadgetIds[x]);
        }

        return urlArray;
    }

    return "NA";
}

UserPrefsService.getUrlArrayFromLayout = function getUrlArrayFromLayout(jsonLayout, gadgetIdStore) {
    var layoutElements = jsonLayout.layout;

    for (var x = 0; x < layoutElements.length; x++) {
        if (layoutElements[x].type == "gadget") {
            gadgetIdStore[gadgetIdStore.length] = layoutElements[x].id
        } else {
            // Container element. Recurse.
            UserPrefsService.getUrlArrayFromLayout(layoutElements[x], gadgetIdStore);
        }
    }
}

UserPrefsService.getTabLayout = function getTabLayout(userId, dashboardName) {
    var response = UserPrefsService.readCookie("tabLayout");
    if (response == "NA") {
        UserPrefsService.createCookie("tabLayout", "0", 0);
        return "0";
    }
    return response;
}

UserPrefsService.getTabTitle = function getTabTitle(userId, tabId,
                                                    dashboardName) {
    if (tabId == 0) {
        return "Home"
    }
    var response = UserPrefsService.readCookie("tab-" + tabId);
    return response;
}

UserPrefsService.removeGadget = function removeGadget(userId, tabId, gadgetId,
                                                      dashboardName) {
    var currentLayout = JSON.parse(UserPrefsService.getGadgetLayout("", tabId, ""));

    currentLayout.layout = UserPrefsService.removeGadgetFromLayoutArray(gadgetId, currentLayout.layout);

    UserPrefsService.setGadgetLayout("", tabId, JSON.stringify(currentLayout), "");

    UserPrefsService.eraseCookie("gadget-" + gadgetId);

    return true;
}

UserPrefsService.removeGadgetFromLayoutArray = function removeGadgetFromLayoutArray(gadgetId, layoutArray) {

    var modifiedLayoutArray = new Array();

    for (var x = 0; x < layoutArray.length; x++) {
        var currentLayoutElement = layoutArray[x];
        if (currentLayoutElement.type == "gadget") {
            if (!(currentLayoutElement.id == gadgetId)) {
                modifiedLayoutArray[modifiedLayoutArray.length] = currentLayoutElement;
            }
        }else{
            // This is a container element
            currentLayoutElement.layout = UserPrefsService.removeGadgetFromLayoutArray(gadgetId, currentLayoutElement.layout);
            modifiedLayoutArray[modifiedLayoutArray.length] = currentLayoutElement;
        }
    }

    return modifiedLayoutArray;
}

UserPrefsService.removeTab = function removeTab(userId, tabId, dashboardName) {
    var curTabLayoutArray = UserPrefsService.getTabLayout("", "").split(",");
    var newLayout = "";
    for (var i = 0; i < curTabLayoutArray.length; i++) {
        if (curTabLayoutArray[i] != tabId) {
            if (newLayout == "") {
                newLayout = curTabLayoutArray[i];
            } else {
                newLayout = newLayout + "," + curTabLayoutArray[i];
            }
        }
    }

    UserPrefsService.createCookie("tabLayout", newLayout, 0);

    UserPrefsService.eraseCookie("tab-" + tabId);
    return true;
}

UserPrefsService.setGadgetLayout = function setGadgetLayout(userId, tabId,
                                                            newLayout, dashboardName) {
    try {
        newLayout = decodeURI(newLayout);
    } catch(e) {
    }

    UserPrefsService.createCookie("gadgetLayout-" + tabId, newLayout, 0);
    return true;
}

UserPrefsService.setGadgetPrefs = function setGadgetPrefs(userId, gadgetId,
                                                          prefId, value, dashboardName) {
    UserPrefsService.createCookie("gadgetPref-" + gadgetId + "-" + prefId, value, 0);
    return true;
}

UserPrefsService.duplicateTab = function duplicateTab(userId, dashboardName, sourceTabId, newTabName)
{
    // Getting the Gadget Layout of the source Tab
    var sourceLayout = JSON.parse(UserPrefsService.getGadgetLayout(userId, sourceTabId, dashboardName));

    // Creating a new Tab
    var newTabId = UserPrefsService.addNewTab(userId, newTabName, dashboardName);

    sourceLayout.layout = UserPrefsService.duplicateGadgetsInLayout(newTabId, sourceLayout.layout);

    UserPrefsService.setGadgetLayout(userId, newTabId, JSON.stringify(sourceLayout), dashboardName);

    return parseInt(newTabId);
}

UserPrefsService.duplicateGadgetsInLayout = function duplicateGadgetsInLayout(tabId, layoutElementsArray) {

    for (var x = 0; x < layoutElementsArray.length; x++) {
        var currentLayoutElement = layoutElementsArray[x];
        if (currentLayoutElement.type == "gadget") {
            var gadgetId = currentLayoutElement.id;
            if (gadgetId) {
                currentLayoutElement.id = UserPrefsService.addGadget(null, tabId, UserPrefsService.getGadgetUrl(gadgetId), null, null);
            } else {
                // This is a container element. Recurse.
                UserPrefsService.duplicateGadgetsInLayout(tabId, currentLayoutElement.layout);
            }
        }
    }

    return layoutElementsArray;
}

UserPrefsService.copyGadget = function copyGadget(userId, tab, dashboardName, sourceGadgetId) {
    UserPrefsService.jsonBuilder();
    //taking the gadget url from cookie
    //var gadgetUrl = UserPrefsService.readCookie("gadget-" + sourceGadgetId);
    var gadgetUrl = UserPrefsService.getGadgetUrl("gadget-" + sourceGadgetId);
    UserPrefsService.addGadget(null, tab, gadgetUrl, null, "G1#");

}

UserPrefsService.moveGadgetToTab = function moveGadgetToTab(userId, tab, dashboardName, sourceGadgetId) {

    //takeing the gadget url from cookie
    //var gadgetUrl = UserPrefsService.readCookie("gadget-" + sourceGadgetId);
    var gadgetUrl = UserPrefsService.getGadgetUrl("gadget-" + sourceGadgetId);
    UserPrefsService.addGadget(null, tab, gadgetUrl, null, "G1#");

}

UserPrefsService.hasTabName = function hasTabName(userId, tabName, dashboardName) {

    //takeing the gadget url from cookie
    var tabIds = UserPrefsService.getTabLayout(userId, dashboardName).split(',');
    for (var x = 0; x < tabIds.length; x++) {
        var tabTitle = UserPrefsService.getTabTitle(userId, tabIds[x], dashboardName);
        if (tabName == tabTitle) {
            return true;
        }
    }
    return false;
}

UserPrefsService.isNewUser = function isNewUser() {
    var curTabId = UserPrefsService.readCookie("nextGadgetId");
    return (curTabId == "NA");
}

UserPrefsService.isValidTab = function isValidTab(userId, currentActiveTab, dashboardName) {
    var title = UserPrefsService.getTabTitle(userId, currentActiveTab, dashboardName);
    if (title == 'NA') {
        return false;
    }

    return true;
}

UserPrefsService.jsonBuilder = function jsonBuilder(gId, gUrl) {
    var gadget = {};
    gadget['gUrl'] = gUrl;
    var gadgets = UserPrefsService.getGadgetJson();
    if (gadgets == null) {
        gadgets = {};
    }
    gadgets[gId] = gadget;
    UserPrefsService.saveGadgetJson(gadgets);
}

UserPrefsService.saveGadgetJson = function saveGadgetJson(gJson) {
    UserPrefsService.createCookie("gadgetJson", JSON.stringify(gJson), 0);
}

UserPrefsService.getGadgetJson = function getGadgetJson() {
    var gadgetJsonText = UserPrefsService.readCookie("gadgetJson");
    if (gadgetJsonText == 'NA') {
        return null;
    } else {
        return JSON.parse(gadgetJsonText);
    }
}

UserPrefsService.getGadgetUrl = function getGadgetUrl(gId) {
    var gadgetJsonObj = UserPrefsService.getGadgetJson();
    return gadgetJsonObj[gId].gUrl;
}

UserPrefsService.populateDefaultThreeColumnLayout = function populateDefaultThreeColumnLayout(userId, tabId) {
    var threeColumnTemplate = layout3ColumnJSON;

    var column1Layout = new Array();
    var column2Layout = new Array();
    var column3Layout = new Array();

    var p = 0;
    for (p = 0; p < gadgetserver.gadgetSpecUrls.length; p = p + 3) {
        var newGadget = new Layout();
        newGadget.type = "gadget";
        newGadget.id = UserPrefsService.storeGadget(gadgetserver.gadgetSpecUrls[p]);

        column1Layout[column1Layout.length] = newGadget;
    }

    threeColumnTemplate.layout[0].layout = column1Layout;

    for (p = 1; p < gadgetserver.gadgetSpecUrls.length; p = p + 3) {
        var newGadget = new Layout();
        newGadget.type = "gadget";
        newGadget.id = UserPrefsService.storeGadget(gadgetserver.gadgetSpecUrls[p]);

        column2Layout[column2Layout.length] = newGadget;
    }

    threeColumnTemplate.layout[1].layout = column2Layout;

    for (p = 2; p < gadgetserver.gadgetSpecUrls.length; p = p + 3) {
        var newGadget = new Layout();
        newGadget.type = "gadget";
        newGadget.id = UserPrefsService.storeGadget(gadgetserver.gadgetSpecUrls[p]);

        column3Layout[column3Layout.length] = newGadget;
    }

    threeColumnTemplate.layout[2].layout = column3Layout;

    // Store this layout
    UserPrefsService.setGadgetLayout(userId, currentActiveTab, JSON.stringify(threeColumnTemplate), null);

    return UserPrefsService.getGadgetLayout(userId, currentActiveTab, null);
}